from pyerrors.input import json as pj
import os
import datalad.api as dl
import sqlite3
from .input import sfcf,openQCD
import json
from typing import Union
from pyerrors import Obs, Corr


def write_measurement(path, ensemble, measurement, uuid, code, parameter_file):
    """
    Write a measurement to the backlog.
    If the file for the measurement already exists, update the measurement.

    Parameters
    ----------
    path: str
        The path to the backlogger folder.
    ensemble: str
        The ensemble of the measurement.
    measurement: dict
        Measurements to be captured in the backlogging system.
    uuid: str
        The uuid of the project.
    """
    if code == "sfcf":
        parameters = sfcf.read_param(path, uuid, parameter_file)
    elif code == "openQCD":
        parameters = openQCD.read_param(path, uuid, parameter_file)
    
    dl.unlock(path + '/backlogger.db', dataset=path)
    conn = sqlite3.connect(path + '/backlogger.db')
    c = conn.cursor()
    files = []
    for corr in measurement.keys():
        file = path + "/archive/" + ensemble + "/" + corr + '/' + uuid + '.json.gz'
        files.append(file)
        known_meas = None
        if not os.path.exists(path + "/archive/" + ensemble + "/" + corr):
            os.makedirs(path + "/archive/" + ensemble + "/" + corr)
        else:
            if os.path.exists(file):
                dl.unlock(file, dataset=path)
                known_meas = pj.load_json_dict(file)
        if code == "sfcf":
            for subkey in measurement[corr].keys():
                meas_path = file + "::" + subkey
                if known_meas is not None:
                    known_meas[subkey] = measurement[corr][subkey]
                    # this should be only set if something changed.
                else:
                    known_meas = measurement[corr]

                if c.execute("SELECT * FROM backlogs WHERE path = ?", (meas_path,)).fetchone() is not None:
                    c.execute("UPDATE backlogs SET updated_at = datetime('now') WHERE path = ?", (meas_path, ))
                else:
                    c.execute("INSERT INTO backlogs (name, ensemble, code, path, project, parameters, parameter_file, created_at, updated_at) VALUES (?, ?, ?, ?, ?, ?, ?, datetime('now'), datetime('now'))", (corr, ensemble, code, meas_path, uuid, sfcf.get_specs(corr + "/" + subkey, parameters), parameter_file))
        elif code == "openQCD":
            for i in range(len(parameters["rw_fcts"])):
                subkey = "/".join(parameters["rw_fcts"][i])
                meas_path = file + "::" + subkey
                c.execute("INSERT INTO backlogs (name, ensemble, code, path, project, parameters, parameter_file, created_at, updated_at) VALUES (?, ?, ?, ?, ?, ?, ?, datetime('now'), datetime('now'))", ("ms1", ensemble, code, meas_path, uuid, json.dumps(parameters["rw_fcts"][i]), parameter_file))
        conn.commit()
        pj.dump_dict_to_json(measurement[corr], file)
    files.append(path + '/backlogger.db')
    conn.close()
    dl.save(files, message="Add measurements to database", dataset=path)


def load_record(path: str, meas_path: str):
    """
    Load a list of records by their paths.

    Parameters
    ----------
    path: str
        Path of the correlator library.
    meas_path: str
        The path to the correlator in the backlog system.
    
    Returns
    -------
    co : Corr or Obs
        The correlator in question.
    """
    return load_records(path, [meas_path])[0]


def load_records(path: str, meas_paths: list[str]) -> list[Union[Corr, Obs]]:
    """
    Load a list of records by their paths.

    Parameters
    ----------
    path: str
        Path of the correlator library.
    meas_paths: list[str]
        A list of the paths to the correlator in the backlog system.
    
    Returns
    -------
    List
    """
    needed_data: dict[str, list[str]] = {}
    for mpath in meas_paths:
        file = path.split("::")[0]
        if file not in needed_data.keys():
            needed_data[file] = []
        key = mpath.split("::")[1]
        needed_data[file].append(key)
    returned_data: list = []
    for filename in needed_data.keys():
        filedict = pj.load_json_dict(filename)
        for key in list(needed_data[filename]):
            returned_data.append(filedict[key])
    return returned_data


def drop_record(path: str, meas_path: str):
    file = meas_path.split("::")[0]
    sub_key = meas_path.split("::")[1]
    dl.unlock(path + '/backlogger.db', dataset=path)
    conn = sqlite3.connect(path + '/backlogger.db')
    c = conn.cursor()
    if c.execute("SELECT * FROM backlogs WHERE path = ?", (meas_path, )).fetchone() is not None:
        c.execute("DELETE FROM backlogs WHERE path = ?", (meas_path, ))
    else:
        raise ValueError("This measurement does not exist as an entry!")

    conn.commit()
    known_meas = pj.load_json_dict(file)
    if sub_key in known_meas:
        del known_meas[sub_key]
        dl.unlock(file, dataset=path)
        pj.dump_dict_to_json(known_meas, file)
        dl.save([path + '/backlogger.db', file], message="Drop measurements to database", dataset=path)
        return
    else:
        raise ValueError("This measurement does not exist as a file!")
